/*
  Programme écrit par Silicium628
  ce logiciel est libre et open source

  */
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "math.h"
#include "complexe.cpp"

QString version = "2.3";

QColor couleur_ecran = QColor::fromRgb(3, 41, 11, 255);
QColor couleur_ligne = QColor::fromRgb(167, 2, 0, 255);
QColor couleur_trace = QColor::fromRgb(0, 210, 0, 255);
QColor couleur_texte = QColor::fromRgb(255, 255, 0, 255);
QColor couleur_signature = QColor::fromRgb(255, 255, 0, 255);

QPen pen_ligne(couleur_ligne, 1, Qt::SolidLine);
QPen pen_trace(couleur_trace, 1, Qt::SolidLine);
QPen pen_reticule(couleur_ligne, 1, Qt::SolidLine);

Complexe ech[1024]; // nb_ech echantillons
Complexe tab_X[1024];   // nb_ech valeurs traitées
Complexe tab_W[512];    // nb_ech/2


float frequence1=400; // Hz
float frequence2=400; // Hz
uint nb_etapes=8;
uint nb_ech = pow (2,nb_etapes); // nombre d'échantillons = 2 puissance(nb_etapes)
float frq_echantillonnage=16*frequence1; // Hz
float Te; // période d'échantillonage

//int pas_balayage=40;
int hamming = false;
int second_Frq = false;
int bz = false;


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)

{
    setupUi(this);
    setWindowTitle("Transformee de Fourier Rapide FFT - version " + version);

    scene = new QGraphicsScene(this);
    scene->setBackgroundBrush(couleur_ecran);
    graphicsView1->setScene(scene);
    graphicsView1->setGeometry(0,0,530,430);

    groupe_reticule = new QGraphicsItemGroup();
    scene->addItem(groupe_reticule);
    groupe_trace = new QGraphicsItemGroup();
    scene->addItem(groupe_trace);

    calcul_tableau_W();
    tracer_graduations();
}




void MainWindow::effacer_graduation()
{
    foreach( QGraphicsItem *item, scene->items( groupe_reticule->boundingRect() ) )
    {
        if( item->group() == groupe_reticule )  { delete item; }
    }
}


void MainWindow::effacer_trace()
{
    foreach( QGraphicsItem *item, scene->items( groupe_trace->boundingRect() ) )
    {
        if( item->group() == groupe_trace )  { delete item; }
    }
}


MainWindow::~MainWindow()
{

}


void MainWindow::tracer_graduations()
{
    float i,x,y;
    float nb_grad_max;
    float intervalle; // séparant les graduations
    float fi;
    QString sti;
    uint decal = 3; // leger decallage vertical pour ne pas masquer la trace

    rectangle = new QGraphicsRectItem(0,decal, 511, 400);
    rectangle->setPen(couleur_ligne);
    groupe_trace->addToGroup(rectangle);

// lignes verticales
// ici calcul de l'intervalle (en pixels) qui correspond exactement à 100Hz (en fréquence)

    intervalle = 2000.0 * nb_ech  * 2.0 / (frq_echantillonnage);
    nb_grad_max = 512 / intervalle;

    for (i=0; i<=nb_grad_max; i++)
    {
        x = intervalle * i;

        ligne1 = new QGraphicsLineItem(x, decal, x, 400 + decal);
        ligne1->setPen(pen_reticule);
        groupe_reticule->addToGroup(ligne1);

        fi = x * frq_echantillonnage  / nb_ech /4;

        sti.setNum(fi);
        texte_frq = new QGraphicsTextItem(sti);
        texte_frq->setDefaultTextColor(couleur_texte);
        texte_frq->setPos(x,380);
        if(x<480) // évite que l'écriture ne déborde du cadre a droite
        {
            groupe_reticule->addToGroup(texte_frq);
        }
    }

// lignes horizontales
    intervalle = 50;
    nb_grad_max = 400 / intervalle;

    for (i=0; i<=nb_grad_max; i++)
    {
        y = 400 + decal - intervalle * i;

        ligne1 = new QGraphicsLineItem(0, y, 511, y);
        ligne1->setPen(pen_reticule);
        groupe_reticule->addToGroup(ligne1);
    }

    texte_frq = new QGraphicsTextItem("Silicium628");
    texte_frq->setDefaultTextColor(couleur_signature);
    texte_frq->setPos(5,5);
    groupe_reticule->addToGroup(texte_frq);

    scene->addItem(groupe_reticule);
}


void MainWindow::tracer_signal()
{
    uint offset_y = 100;
    float echelle =1024/nb_ech;
    uint n;
    float x, y, memo_x, memo_y;

    segment_trace = new QGraphicsLineItem(0, offset_y, 512, offset_y);
    segment_trace->setPen(couleur_ligne);
    groupe_trace->addToGroup(segment_trace);

    effacer_trace();

    x=0;
    y=offset_y;
    for(n=0; n<nb_ech; n++)
    {
        memo_x = x;
        memo_y = y;
        x =  echelle * n /2;

        y = offset_y - 20 * ech[n].a;

        segment_trace = new QGraphicsLineItem(memo_x ,memo_y, x, y);
        segment_trace->setPen(pen_trace);
        groupe_trace->addToGroup(segment_trace);
    }
    scene->addItem(groupe_trace);
}



void MainWindow::tracer_tableau_valeurs()
{
    uint offset_y = 350; // 600
    float echelle =1024/nb_ech;
    uint n;
    float x, y, memo_x, memo_y;

    int z;

    if (bz) {z=4;} else {z=1;}

    segment_trace = new QGraphicsLineItem(0, offset_y, 512, offset_y);
    segment_trace->setPen(couleur_ligne);
    groupe_trace->addToGroup(segment_trace);



    x=0;
    y=offset_y;
    for(n=0; n<nb_ech/2; n++)
    {
        memo_x = x;
        memo_y = y;
        x =  echelle * n;

        y = offset_y - z * sqrt( tab_X[n].a * tab_X[n].a + tab_X[n].b * tab_X[n].b );

        segment_trace = new QGraphicsLineItem(memo_x ,memo_y, x, y);
        segment_trace->setPen(pen_trace);
        groupe_trace->addToGroup(segment_trace);
    }
    scene->addItem(groupe_trace);
}



void MainWindow::RAZ_tableau_echantillons()
{
   uint n;
   for(n=0; n < nb_ech; n++)
   {
       ech[n].a = 0; // partie reelle
       ech[n].b = 0; // partie imaginaire
   }
}




void MainWindow::remplir_tableau_echantillons()
{
// création du signal à analyser
    uint n, n_max;
    float x;

    Te = 1.0 / frq_echantillonnage;

    if (bz==true) {n_max = nb_ech /4; } else {n_max = nb_ech; }

    for(n=0; n < n_max; n++) // nb d'échantillons
    {
        x=  cos(2.0 * M_PI * frequence1 * n * Te );
        if (second_Frq) {x+=cos(2 * M_PI * frequence2 * n * Te); }
        if (hamming)  { x = x * (1- cos (2* M_PI * n / n_max ));}
        ech[n].a = x; // partie reelle
        ech[n].b = 0; // partie imaginaire
    }
}


//void MainWindow::remplir_tableau_echantillons()
//{
//// création du signal à analyser
//    uint n;
//    float x;

//    Te = 1.0 / frq_echantillonnage;

//    for(n=0; n < nb_ech; n++) // nb d'échantillons
//    {
//        if ((n >= 50) and (n<=55)) {ech[n].a = 30;} else {ech[n].a = 0;}

//    }
//}


uint bit_reversal(uint num, uint nb_bits)
{
    uint r = 0, i, s;
    if ( num > 1<< nb_bits) { return 0; }

    for (i=0; i<nb_bits; i++)
    {
        s = (num & (1 << i));
        if(s) { r |= (1 << ((nb_bits - 1) - i)); }
    }
    return r;
}



void MainWindow::bit_reverse_tableau_X()
{

// recopie les échantillons en les classant dans l'ordre 'bit reverse'
    uint n,r;

    for(n=0; n < nb_ech; n++) // nb d'échantillons
    {
        r=bit_reversal(n,nb_etapes);
        tab_X[n] = ech[r];
    }
}



void MainWindow::calcul_tableau_W()
{
// calcul et memorisation dans un tableau des twiddle factors
    uint n;
    float x;

    for(n=0; n<(nb_ech/2-1); n++)
    {
        x=2.0*M_PI * n / nb_ech;

        tab_W[n].a = cos(x);  // partie reelle
        tab_W[n].b = -sin(x); // partie imaginaire
    }
}



void MainWindow::calcul_fourier()
{
    Complexe produit; // voir la classe "Complexe" : complexe.h et complexe.ccp
    uint decal =2;

    uint etape, e1, e2, li, w, ci;
    uint li_max=nb_ech;

    e2=1;

    for (etape=1; etape<=nb_etapes; etape++)
    {
        e1=e2; //(e1 evite d'effectuer des divisions e2/2 plus bas)
        e2=e2+e2;

        for (li=0; li<li_max; li+=1)
        {
            ci=li & (e2-1);  // ET bit à bit
            if (ci>(e1-1))
            {
                w=li_max/e2*(li & (e1 -1));  // ET bit à bit  calcul du numéro du facteur de tripatouillages W

                produit = tab_W[w] * tab_X[li]; // le twiddle factor est lu en memoire; le produit est une mutiplication de nb complexes. Voir "complexe.cpp"

                tab_X[li]=tab_X[li-e1]-produit;         // concerne la ligne basse du croisillon; soustraction complexe; Voir "complexe.cpp"
                tab_X[li-e1]=tab_X[li-e1] + produit;    // concerne la ligne haute du croisillon; addition complexe; Voir "complexe.cpp"
            }
        }
    }
}



void MainWindow::on_btn2_clicked() // BOUTON "FOURIER"
{
   effacer_trace();
   calcul_tableau_W();
   RAZ_tableau_echantillons();
   remplir_tableau_echantillons();
   tracer_signal();
   bit_reverse_tableau_X();
   calcul_fourier();
   tracer_graduations();
   tracer_tableau_valeurs();
}



void MainWindow::on_spinBox_frq_valueChanged(int arg1) // SELECTEUR DE FREQUENCE 1
{
   effacer_trace();
   frequence1 = arg1;
   RAZ_tableau_echantillons();
   remplir_tableau_echantillons();
   tracer_signal();
   calcul_tableau_W();
   bit_reverse_tableau_X();
   calcul_fourier();
  //  tracer_graduations();
    tracer_tableau_valeurs();
}


void MainWindow::on_spinBox_frq_2_valueChanged(int arg1) // SELECTEUR DE FREQUENCE 2
{
    effacer_trace();
    frequence2 = arg1;
    RAZ_tableau_echantillons();
    remplir_tableau_echantillons();
    tracer_signal();
    calcul_tableau_W();
    bit_reverse_tableau_X();
    calcul_fourier();
   //  tracer_graduations();
    tracer_tableau_valeurs();
}



void MainWindow::on_checkBox1_toggled(bool checked) // HAMMING
{
   effacer_trace();
   if (checked) {hamming = true;} else {hamming = false;}
   RAZ_tableau_echantillons();
   remplir_tableau_echantillons();
   tracer_signal();
   calcul_tableau_W();
   bit_reverse_tableau_X();
   calcul_fourier();
   // tracer_graduations();
   tracer_tableau_valeurs();

}


void MainWindow::on_checkBox2_toggled(bool checked)  // SECONDE FREQUENCE
{
    effacer_trace();
    if (checked) {second_Frq = true;} else {second_Frq = false;}
    RAZ_tableau_echantillons();
    remplir_tableau_echantillons();
    tracer_signal();
    calcul_tableau_W();
    bit_reverse_tableau_X();
    calcul_fourier();
   // tracer_graduations();
    tracer_tableau_valeurs();
}


void MainWindow::on_checkBox3_toggled(bool checked)
{
    effacer_trace();
    if (checked) {bz = true;} else {bz = false;}
    RAZ_tableau_echantillons();
    remplir_tableau_echantillons();
    tracer_signal();
    calcul_tableau_W();
    bit_reverse_tableau_X();
    calcul_fourier();
   // tracer_graduations();
    tracer_tableau_valeurs();
}
